import JaCoP.core.IntVar;
import JaCoP.search.*;
import SoftLib.constraints.*;
import SoftLib.core.SoftStore;
import SoftLib.weight.*;

public class Esempio_softArcConsistency_graph_shaped_CSP {
    public static void main(String[] args) {

        SoftStore softStore = new SoftStore(SoftStore.Semiring.FUZZY);
        
        int varNum = 70;
        int localDomainMin = 1;
        int localDomainMax = 5;
        
        
        IntVar[] list = new IntVar[varNum];
        SoftXneqC xneqcCon;
        for(int i=0; i< list.length; i++){
            list[i] = new IntVar(softStore, "X"+i, localDomainMin, localDomainMax);
            xneqcCon = new SoftXneqC(list[i], i%localDomainMax);
            xneqcCon.impose(softStore, new WeightUnaryDistance(localDomainMax/2));
        }
        
        //different/lesser values for a distance = 1 in the variables list
        SoftXneqY xnycon;
        SoftXltY xltycon;
        for(int i=1; i<list.length; i++){
            if(i%3!=0){
                xnycon = new SoftXneqY(list[i-1], list[i]);
                xnycon.impose(softStore, new WeightMax());
            }else{
                xltycon = new SoftXltY(list[i-1], list[i]);
                xltycon.impose(softStore, new WeightMin());
            }
        }
        
        //different values for a distance = 2 in the variables list
        SoftXneqY xneqyCon;
        for(int i=2; i<list.length; i++){
            xneqyCon = new SoftXneqY(list[i-2], list[i]);
            xneqyCon.impose(softStore, new WeightDistance());
        }
        
        //different values for a distance = 3 in the variables list
        for(int i=3; i<list.length; i++){
            xneqyCon = new SoftXneqY(list[i-3], list[i]);
            xneqyCon.impose(softStore, new WeightDistance());
        }
        
        long beforeSoftArcConsistencyTime = System.currentTimeMillis();

        //softStore.softArcConsistency();
        softStore.imposeWeights();

        long searchStartTime = System.currentTimeMillis();

        Search<IntVar> label = new DepthFirstSearch<IntVar>();
        SelectChoicePoint<IntVar> select = new SimpleSelect<IntVar>(list,
                new SmallestDomain<IntVar>(),
                new IndomainMin<IntVar>());

        label.getSolutionListener().searchAll(true);
        label.getSolutionListener().recordSolutions(true);

        label.labeling(softStore, select, softStore.getCost());

        long searchEndTime = System.currentTimeMillis();

        System.out.println("softArcConsistency execution time: "+(searchStartTime - beforeSoftArcConsistencyTime));
        System.out.println("search execution time: "+(searchEndTime - searchStartTime));
        System.out.println("total execution time: "+(searchEndTime - beforeSoftArcConsistencyTime)+"\n");

        for (int i = 1; i <= label.getSolutionListener().solutionsNo(); i++) {
            System.out.print("Soluzione " + i + ": ");
            for (int j = 0; j < label.getSolution(i).length; j++) {
                System.out.print(" "+label.getSolution(i)[j]);
            }
            System.out.println();
        }
    }
}