package SoftLib.weight;

import JaCoP.constraints.Constraint;
import JaCoP.constraints.Distance;
import JaCoP.core.IntVar;
import SoftLib.core.SoftStore;

/**
 * Weight function used to impose the distance of the variable value from
 * a constant as the variable weight.
 */

public class WeightUnaryDistance extends PrimitiveUnaryWeight{

    int c;

    public WeightUnaryDistance(int c) {
        this.c = c;
    }
    
    @Override
    public Constraint setWeight(SoftStore store, IntVar x) {
        
        int min = (c >= x.min() && c <= x.max()) ? 0 : (Math.min( Math.abs(x.min()-c), Math.abs(x.max()-c)));
        int max = Math.max( Math.abs(x.min()-c), Math.abs(x.max()-c));
        if((store.getSrType() == SoftStore.Semiring.FUZZY ||
            store.getSrType() == SoftStore.Semiring.PROBABILISTIC) &&
            (max > 1000 || min < 0)) {
            throw new UnsupportedOperationException("In " + ((store.getSrType() == SoftStore.Semiring.FUZZY) ? "Fuzzy" : "Probabilistic") + " semiring the probability/preference value of an assignment cannot be less than 0 or greater than 1000");
        }
        IntVar z = new IntVar(store, "store.cost", min, max);
        store.costs.add(z);
        IntVar y = new IntVar(store, c, c);
        Constraint cost = new Distance(x, y, z);
        return cost;
    }

    @Override
    public int evaluateWeight(int x) {
        return Math.abs(x-c);
    }
}
