package SoftLib.weight;

import JaCoP.constraints.Constraint;
import JaCoP.constraints.ExtensionalSupportSTR;
import JaCoP.core.IntVar;
import JaCoP.core.Interval;
import SoftLib.core.SoftStore;
import java.util.HashMap;

/**
 * This class permitts to specify, for each value assigned to the variable, the related weight.
 * To use this class it is necessary to specify the weight/preference/probability value for each 
 * variables domain value 
 */

public class WeightUnaryExtensional extends PrimitiveUnaryWeight{

    private int[] w;
    private HashMap<Integer, Integer> values;
    
    public WeightUnaryExtensional(int[] w) {
        this.w = w;
        this.values = new HashMap<Integer, Integer>();
    }
    
    @Override
    public Constraint setWeight(SoftStore store, IntVar x) {
        int zDomSize = x.getSize();
        if(w.length != zDomSize){
            throw new IllegalArgumentException("Wrong weights dimension");
        }
        int min, max;
        min = w[0];
        max = w[0];
        for(int i=1; i<w.length; i++){
            if(w[i] < min){min = w[i];}
            if(w[i] > max){max = w[i];}
        }
        if((store.getSrType() == SoftStore.Semiring.FUZZY ||
            store.getSrType() == SoftStore.Semiring.PROBABILISTIC) &&
            (max > 1000 || min < 0)) {
            throw new UnsupportedOperationException("In " + ((store.getSrType() == SoftStore.Semiring.FUZZY) ? "Fuzzy" : "Probabilistic") + " semiring the probability/preference value of an assignment cannot be less than 0 or greater than 1000");
        }
        int[][] tuples = new int[zDomSize][2];
        Interval current;
        int i=0, index=0;
        while((current = x.domain.getInterval(i++)) != null){
            for(int value = current.min(); value <= current.max(); value++){
                tuples[index][0] = value;
                tuples[index][1] = w[index];
                values.put(value, w[index]);
                index++;
            }
        }
        IntVar z = new IntVar(store, min, max);
        store.costs.add(z);
        IntVar[] v = {x, z};        
        Constraint cost = new ExtensionalSupportSTR(v, tuples);
        return cost;
    }

    @Override
    public int evaluateWeight(int x) {
        if(values.containsKey(x)){
            return values.get(x);
        } else{
            throw new IllegalArgumentException("x value is not in the variable domain");
        }
    }
    
}