package SoftLib.weight;

import JaCoP.constraints.Constraint;
import JaCoP.constraints.XmulCeqZ;
import JaCoP.core.IntVar;
import SoftLib.core.SoftStore;

/**
 * Weight function used to impose a proportionality between the variable value
 * and its weight.
 */

public class WeightUnaryProportional extends PrimitiveUnaryWeight{
 
    int c;
    
    public WeightUnaryProportional(int c){
        this.c = c;
    }
    
    @Override
    public Constraint setWeight(SoftStore store, IntVar x) {
        
        int min = x.min()*c;
        int max = x.max()*c;
        if((store.getSrType() == SoftStore.Semiring.FUZZY ||
            store.getSrType() == SoftStore.Semiring.PROBABILISTIC) &&
            (max > 1000 || min < 0)) {
            throw new UnsupportedOperationException("In " + ((store.getSrType() == SoftStore.Semiring.FUZZY) ? "Fuzzy" : "Probabilistic") + " semiring the probability/preference value of an assignment cannot be less than 0 or greater than 1000");
        }

        IntVar z = new IntVar(store, "store.cost", min, max);
        store.costs.add(z);
        Constraint cost = new XmulCeqZ(x, c, z);
        return cost;
    }

    @Override
    public int evaluateWeight(int x) {
        return x*c;
    }
}